"""Tests for CLIMCPServer functionality."""

from unittest.mock import Mock, patch
from mcp_chain.cli_mcp import CLIMCPServer


def test_cli_mcp_server_creation():
    """Test creating a CLIMCPServer."""
    server = CLIMCPServer("test-cli", commands=["ls"])
    
    assert server.name == "test-cli"
    assert server.commands == ["ls"]


def test_cli_mcp_server_creation_with_description():
    """Test creating a CLIMCPServer with custom description."""
    custom_desc = "Custom description for ls command"
    server = CLIMCPServer("test-cli", commands=["ls"], descriptions={"ls": custom_desc})
    
    assert server.name == "test-cli"
    assert server.commands == ["ls"]
    assert server.descriptions == {"ls": custom_desc}


def test_cli_mcp_server_implements_dict_mcp_server_protocol():
    """Test that CLIMCPServer implements DictMCPServer protocol."""
    server = CLIMCPServer("test-cli", commands=["echo"])
    
    # Should have get_metadata method
    assert hasattr(server, 'get_metadata')
    assert callable(server.get_metadata)
    
    # Should have handle_request method
    assert hasattr(server, 'handle_request')
    assert callable(server.handle_request)


def test_get_metadata_returns_tools():
    """Test that get_metadata returns tools for configured command."""
    server = CLIMCPServer("test-cli", commands=["echo"])
    
    metadata = server.get_metadata()
    
    assert isinstance(metadata, dict)
    assert "tools" in metadata
    assert "server_name" in metadata
    assert metadata["server_name"] == "test-cli"
    
    # Should have one tool for echo command
    tools = metadata["tools"]
    assert len(tools) == 1
    assert tools[0]["name"] == "echo"


def test_handle_tools_list_request():
    """Test handling tools/list request."""
    server = CLIMCPServer("test-cli", commands=["echo"])
    
    request = {
        "jsonrpc": "2.0",
        "id": 1,
        "method": "tools/list",
        "params": {}
    }
    
    response = server.handle_request(request)
    
    assert response["jsonrpc"] == "2.0"
    assert response["id"] == 1
    assert "result" in response
    assert "tools" in response["result"]
    assert len(response["result"]["tools"]) == 1


@patch('subprocess.run')
def test_handle_tool_call_request(mock_run):
    """Test handling tools/call request."""
    # Mock subprocess.run
    mock_result = Mock()
    mock_result.stdout = "Hello World"
    mock_result.stderr = ""
    mock_result.returncode = 0
    mock_run.return_value = mock_result
    
    server = CLIMCPServer("test-cli", commands=["echo"])
    
    request = {
        "jsonrpc": "2.0",
        "id": 2,
        "method": "tools/call",
        "params": {
            "name": "echo",
            "arguments": {
                "_args": ["Hello World"]
            }
        }
    }
    
    response = server.handle_request(request)
    
    assert response["jsonrpc"] == "2.0"
    assert response["id"] == 2
    assert "result" in response
    assert not response["result"]["isError"]
    assert len(response["result"]["content"]) == 1
    assert "Hello World" in response["result"]["content"][0]["text"]
    
    # Verify subprocess.run was called correctly
    mock_run.assert_called_once()
    call_args = mock_run.call_args[0][0]  # First positional argument
    assert call_args[0] == "echo"
    assert "Hello World" in call_args


@patch('subprocess.run')
def test_handle_tool_call_with_flags(mock_run):
    """Test handling tools/call request with boolean flags."""
    mock_result = Mock()
    mock_result.stdout = "output"
    mock_result.stderr = ""
    mock_result.returncode = 0
    mock_run.return_value = mock_result
    
    server = CLIMCPServer("test-cli", commands=["ls"])
    
    request = {
        "jsonrpc": "2.0",
        "id": 3,
        "method": "tools/call",
        "params": {
            "name": "ls",
            "arguments": {
                "l": True,  # Should become -l
                "a": True,  # Should become -a
                "verbose": True  # Should become --verbose
            }
        }
    }
    
    response = server.handle_request(request)
    
    assert response["jsonrpc"] == "2.0"
    assert response["id"] == 3
    assert "result" in response
    
    # Verify subprocess.run was called with correct flags
    mock_run.assert_called_once()
    call_args = mock_run.call_args[0][0]  # First positional argument
    assert call_args[0] == "ls"
    assert "-l" in call_args
    assert "-a" in call_args
    assert "--verbose" in call_args


def test_handle_unknown_method():
    """Test handling unknown method request."""
    server = CLIMCPServer("test-cli", commands=["echo"])
    
    request = {
        "jsonrpc": "2.0",
        "id": 4,
        "method": "unknown/method",
        "params": {}
    }
    
    response = server.handle_request(request)
    
    assert response["jsonrpc"] == "2.0"
    assert response["id"] == 4
    assert "error" in response
    assert response["error"]["code"] == -32601
    assert "Method not found" in response["error"]["message"]


def test_handle_unknown_tool():
    """Test handling call to unknown tool."""
    server = CLIMCPServer("test-cli", commands=["echo"])
    
    request = {
        "jsonrpc": "2.0",
        "id": 5,
        "method": "tools/call",
        "params": {
            "name": "unknown_command",
            "arguments": {}
        }
    }
    
    response = server.handle_request(request)
    
    assert response["jsonrpc"] == "2.0"
    assert response["id"] == 5
    assert "error" in response
    assert response["error"]["code"] == -32602
    assert "Tool not found" in response["error"]["message"]


@patch('subprocess.run')
def test_command_error_handling(mock_run):
    """Test handling command execution errors."""
    # Mock subprocess.run to raise an exception
    mock_run.side_effect = Exception("Command failed")
    
    server = CLIMCPServer("test-cli", commands=["echo"])
    
    request = {
        "jsonrpc": "2.0",
        "id": 6,
        "method": "tools/call",
        "params": {
            "name": "echo",
            "arguments": {"_args": ["test"]}
        }
    }
    
    response = server.handle_request(request)
    
    assert response["jsonrpc"] == "2.0"
    assert response["id"] == 6
    assert "result" in response
    assert response["result"]["isError"]
    assert "Error executing echo" in response["result"]["content"][0]["text"]


@patch('subprocess.run')
def test_command_with_stderr_and_exit_code(mock_run):
    """Test handling command with stderr output and non-zero exit code."""
    mock_result = Mock()
    mock_result.stdout = "some output"
    mock_result.stderr = "error message"
    mock_result.returncode = 1
    mock_run.return_value = mock_result
    
    server = CLIMCPServer("test-cli", commands=["ls"])
    
    request = {
        "jsonrpc": "2.0",
        "id": 7,
        "method": "tools/call",
        "params": {
            "name": "ls",
            "arguments": {"_args": ["/nonexistent"]}
        }
    }
    
    response = server.handle_request(request)
    
    assert response["jsonrpc"] == "2.0"
    assert response["id"] == 7
    assert "result" in response
    assert not response["result"]["isError"]  # We still return success but include error info
    
    content = response["result"]["content"][0]["text"]
    assert "STDOUT:" in content
    assert "some output" in content
    assert "STDERR:" in content
    assert "error message" in content
    assert "Exit code: 1" in content


def test_single_command_server():
    """Test CLIMCPServer with a single command in the list."""
    server = CLIMCPServer("test-cli", commands=["echo"])
    
    metadata = server.get_metadata()
    
    assert len(metadata["tools"]) == 1
    assert metadata["tools"][0]["name"] == "echo"
    assert metadata["server_name"] == "test-cli"


@patch('subprocess.run')
def test_get_help_text_integration(mock_run):
    """Test that help text extraction works with real-like subprocess calls."""
    # Mock the help command call
    mock_result = Mock()
    mock_result.stdout = """ls - list directory contents

Usage: ls [OPTION]... [FILE]...
List information about the FILEs (the current directory by default).

Options:
  -a, --all                  do not ignore entries starting with .
  -l                         use a long listing format
  --help                     display this help and exit
"""
    mock_result.stderr = ""
    mock_result.returncode = 0
    mock_run.return_value = mock_result
    
    server = CLIMCPServer("test-cli", commands=["ls"])
    
    # This should trigger help text extraction
    metadata = server.get_metadata()
    
    assert len(metadata["tools"]) == 1
    tool = metadata["tools"][0]
    assert tool["name"] == "ls"
    assert "list directory contents" in tool["description"]
    
    # Should have extracted some options
    schema = tool["inputSchema"]
    assert "properties" in schema
    assert "_args" in schema["properties"]  # Positional args always included 


@patch('subprocess.run')
def test_description_override_functionality(mock_run):
    """Test that custom description overrides help text extraction."""
    # Mock help text extraction
    mock_result = Mock()
    mock_result.stdout = "ls - list directory contents\nUsage: ls [OPTION]... [FILE]..."
    mock_result.stderr = ""
    mock_result.returncode = 0
    mock_run.return_value = mock_result
    
    custom_desc = "My custom description for ls tool"
    server = CLIMCPServer("test-cli", commands=["ls"], descriptions={"ls": custom_desc})
    
    metadata = server.get_metadata()
    
    # Should use custom description instead of extracted one
    assert len(metadata["tools"]) == 1
    tool = metadata["tools"][0]
    assert tool["name"] == "ls"
    assert tool["description"] == custom_desc
    
    # Verify help text was still fetched (for input schema)
    mock_run.assert_called()


@patch('subprocess.run')
def test_fallback_to_help_text_when_no_override(mock_run):
    """Test that help text is used when no description override is provided."""
    # Mock help text extraction
    mock_result = Mock()
    mock_result.stdout = "ls - list directory contents\nUsage: ls [OPTION]... [FILE]..."
    mock_result.stderr = ""
    mock_result.returncode = 0
    mock_run.return_value = mock_result
    
    server = CLIMCPServer("test-cli", commands=["ls"])  # No descriptions parameter
    
    metadata = server.get_metadata()
    
    # Should extract description from help text
    assert len(metadata["tools"]) == 1
    tool = metadata["tools"][0]
    assert tool["name"] == "ls"
    assert tool["description"] == "ls - list directory contents"
    
    # Verify help text was fetched
    mock_run.assert_called()


def test_cli_mcp_server_creation_with_multiple_commands():
    """Test creating a CLIMCPServer with multiple commands."""
    commands = ["git", "docker", "ls"]
    server = CLIMCPServer("multi-cli", commands=commands)
    
    assert server.name == "multi-cli"
    assert server.commands == commands
    assert len(server.commands) == 3
    assert "git" in server.commands
    assert "docker" in server.commands
    assert "ls" in server.commands 


def test_cli_mcp_server_creation_with_descriptions_dict():
    """Test creating a CLIMCPServer with multiple commands and custom descriptions."""
    commands = ["git", "docker"]
    descriptions = {
        "git": "Custom Git description",
        "docker": "Custom Docker description"
    }
    server = CLIMCPServer("multi-cli", commands=commands, descriptions=descriptions)
    
    assert server.name == "multi-cli"
    assert server.commands == commands
    assert server.descriptions == descriptions
    assert server.descriptions["git"] == "Custom Git description"
    assert server.descriptions["docker"] == "Custom Docker description" 


def test_get_metadata_with_multiple_commands():
    """Test that get_metadata returns multiple tools for multiple commands."""
    commands = ["echo", "ls", "date"]
    server = CLIMCPServer("multi-cli", commands=commands)
    
    metadata = server.get_metadata()
    
    assert len(metadata["tools"]) == 3
    tool_names = [tool["name"] for tool in metadata["tools"]]
    assert "echo" in tool_names
    assert "ls" in tool_names
    assert "date" in tool_names
    assert metadata["server_name"] == "multi-cli" 


@patch('subprocess.run')
def test_descriptions_dict_overrides_in_metadata(mock_run):
    """Test that descriptions dict overrides tool descriptions in metadata."""
    # Mock help text for commands
    mock_result = Mock()
    mock_result.stdout = "echo - display a line of text\nUsage: echo [OPTION]... [STRING]..."
    mock_result.stderr = ""
    mock_result.returncode = 0
    mock_run.return_value = mock_result
    
    commands = ["echo", "ls"]
    descriptions = {
        "echo": "Custom echo tool description",
        "ls": "Custom ls tool description"
    }
    server = CLIMCPServer("multi-cli", commands=commands, descriptions=descriptions)
    
    metadata = server.get_metadata()
    
    assert len(metadata["tools"]) == 2
    
    # Find tools by name and check descriptions
    tools_by_name = {tool["name"]: tool for tool in metadata["tools"]}
    
    assert "echo" in tools_by_name
    assert "ls" in tools_by_name
    assert tools_by_name["echo"]["description"] == "Custom echo tool description"
    assert tools_by_name["ls"]["description"] == "Custom ls tool description" 


@patch('subprocess.run')
def test_handle_tool_call_with_multiple_commands(mock_run):
    """Test handling tool calls for specific commands when multiple commands are available."""
    # Mock subprocess.run to capture the executed command
    mock_result = Mock()
    mock_result.stdout = "test output"
    mock_result.stderr = ""
    mock_result.returncode = 0
    mock_run.return_value = mock_result
    
    commands = ["echo", "ls", "date"]
    server = CLIMCPServer("multi-cli", commands=commands)
    
    # Test calling specific command (ls)
    request = {
        "jsonrpc": "2.0",
        "id": 1,
        "method": "tools/call",
        "params": {
            "name": "ls",
            "arguments": {"_args": ["-l"]}
        }
    }
    
    response = server.handle_request(request)
    
    assert response["jsonrpc"] == "2.0"
    assert response["id"] == 1
    assert "result" in response
    assert not response["result"]["isError"]
    assert "test output" in response["result"]["content"][0]["text"]
    
    # Verify the correct command was executed
    mock_run.assert_called_with(
        ["ls", "-l"],
        capture_output=True,
        text=True,
        timeout=30,
        check=False
    ) 


#!/usr/bin/env python3
"""Example MCP chain using CLIMCPServer with a CLI command."""

from mcp_chain import CLIMCPServer, mcp_chain

# Create a CLI server for the ls command with custom description
cli_server = CLIMCPServer(
    name="ls-tool",
    commands=["ls"],
    descriptions={
        "ls": "List files and directories in the current path with detailed information"
    }
)

# Create the chain - for now just the CLI server
# You could add middleware transformers here too
chain = mcp_chain().then(cli_server)

# The chain variable will be auto-detected by mcp-chain CLI 


"""FastMCP adapter for MCP Chain."""

import logging
from typing import Dict, Any, Set
from mcp.server.fastmcp import FastMCP
from .types import DictMCPServer

logger = logging.getLogger(__name__)


class FastMCPServer:
    """Adapter that uses FastMCP to delegate to dict-based middleware chain."""
    
    def __init__(self, downstream_server: DictMCPServer, name: str = "mcp-chain"):
        """Initialize FastMCPServer with downstream middleware chain.
        
        Args:
            downstream_server: The dict-based MCP server to wrap
            name: Name of the MCP server
        """
        self._downstream = downstream_server
        self._name = name
        self._version = "1.9.3"
        
        # Get metadata and handle errors
        try:
            metadata = self._downstream.get_metadata()
        except Exception as e:
            logger.error("Failed to retrieve metadata from downstream server: %s", e)
            raise RuntimeError("Metadata retrieval failed") from e
        
        # Initialize FastMCP
        try:
            self._fastmcp = FastMCP(name)
        except Exception as e:
            logger.error("Failed to initialize FastMCP: %s", e)
            raise
        
        # Register tools and resources with duplicate detection
        self._register_tools_and_resources(metadata)
    
    def _register_tools_and_resources(self, metadata: Dict[str, Any]):
        """Register tools and resources from metadata with duplicate detection."""
        tools = metadata.get("tools", [])
        resources = metadata.get("resources", [])
        
        # Log if no tools or resources found
        if not tools:
            logger.info("No tools found in metadata")
        if not resources:
            logger.info("No resources found in metadata")
        
        # Track registered names/URIs for duplicate detection
        registered_tool_names: Set[str] = set()
        registered_resource_uris: Set[str] = set()
        
        # Register tools with duplicate detection
        for tool_data in tools:
            if not isinstance(tool_data, dict):
                logger.error("Malformed tool metadata: expected dict, got %s", type(tool_data))
                continue
            
            tool_name = tool_data.get("name")
            if not tool_name:
                logger.error("Malformed tool metadata: missing 'name' field")
                continue
            
            if tool_name in registered_tool_names:
                logger.warning("Duplicate tool name '%s' found, skipping registration", tool_name)
                continue
            
            registered_tool_names.add(tool_name)
            self._register_tool(tool_data)
        
        # Register resources with duplicate detection
        for resource_data in resources:
            if not isinstance(resource_data, dict):
                logger.error("Malformed resource metadata: expected dict, got %s", type(resource_data))
                continue
            
            resource_uri = resource_data.get("uri")
            if not resource_uri:
                logger.error("Malformed resource metadata: missing 'uri' field")
                continue
            
            if resource_uri in registered_resource_uris:
                logger.warning("Duplicate resource URI '%s' found, skipping registration", resource_uri)
                continue
            
            registered_resource_uris.add(resource_uri)
            self._register_resource(resource_data)
    
    def _register_tool(self, tool_data: Dict[str, Any]):
        """Register a single tool with FastMCP."""
        tool_name = tool_data["name"]
        tool_description = tool_data.get("description", f"Execute {tool_name} tool")
        
        # Create a function that will be registered with FastMCP
        def create_tool_function():
            def tool_function(arguments: dict = None):
                """Dynamic tool handler that delegates to downstream server."""
                if arguments is None:
                    arguments = {}
                
                request = {
                    "method": "tools/call",
                    "id": 1,
                    "params": {
                        "name": tool_name,
                        "arguments": arguments
                    }
                }
                try:
                    response = self._downstream.handle_request(request)
                    return response
                except Exception as e:
                    logger.error("Error executing tool '%s': %s", tool_name, e)
                    raise
            
            # Set the function metadata for FastMCP
            tool_function.__name__ = tool_name
            tool_function.__doc__ = tool_description
            return tool_function
        
        # Register the tool with FastMCP using the decorator pattern
        tool_func = create_tool_function()
        # Use the decorator pattern instead of calling the method directly
        decorated_func = self._fastmcp.tool()(tool_func)
        
        # Store the decorated function to ensure it's not garbage collected
        if not hasattr(self, '_registered_tools'):
            self._registered_tools = {}
        self._registered_tools[tool_name] = decorated_func
        
        # Log successful tool registration
        logger.info("Successfully registered tool '%s' with description: %s", tool_name, tool_description)
    
    def _register_resource(self, resource_data: Dict[str, Any]):
        """Register a single resource with FastMCP."""
        resource_uri = resource_data["uri"]
        
        @self._fastmcp.resource(resource_uri)
        def resource_handler():
            """Dynamic resource handler that delegates to downstream server."""
            request = {
                "method": "resources/read",
                "id": 1,
                "params": {
                    "uri": resource_uri
                }
            }
            try:
                response = self._downstream.handle_request(request)
                return response
            except Exception as e:
                logger.error("Error reading resource '%s': %s", resource_uri, e)
                raise
        
        # Set the function name for FastMCP
        resource_handler.__name__ = f"resource_{resource_uri.replace('://', '_').replace('/', '_')}"
    
    def run(self, **kwargs):
        """Start the MCP server.
        
        Args:
            **kwargs: Arguments passed to FastMCP.run()
        """
        # Filter out 'name' parameter since it's handled in __init__
        filtered_kwargs = {k: v for k, v in kwargs.items() if k != 'name'}
        return self._fastmcp.run(**filtered_kwargs)


"""Tests for FastMCPServer adapter."""

from typing import Dict, Any

from mcp_chain.fastmcp import FastMCPServer


class MockDictMCPServer:
    """Mock DictMCPServer for testing."""
    
    def __init__(self, metadata: Dict[str, Any] = None):
        if metadata is None:
            self._metadata = {
                "tools": [
                    {"name": "test_tool", "description": "A test tool"}
                ],
                "resources": [
                    {"uri": "test://resource", "name": "test_resource"}
                ]
            }
        else:
            self._metadata = metadata
    
    def get_metadata(self) -> Dict[str, Any]:
        return self._metadata
    
    def handle_request(self, request: Dict[str, Any]) -> Dict[str, Any]:
        return {"result": f"Handled: {request.get('method', 'unknown')}"}


def test_fastmcp_server_initialization():
    """Test FastMCPServer can be initialized with a DictMCPServer."""
    mock_server = MockDictMCPServer()
    
    # This should create a FastMCPServer instance
    fastmcp_server = FastMCPServer(mock_server)
    
    assert fastmcp_server is not None
    assert fastmcp_server._downstream == mock_server


def test_fastmcp_server_dynamic_tool_registration():
    """Test that FastMCPServer registers tools from downstream metadata."""
    mock_server = MockDictMCPServer({
        "tools": [
            {"name": "query_db", "description": "Query the database"},
            {"name": "update_record", "description": "Update a record"}
        ]
    })
    
    fastmcp_server = FastMCPServer(mock_server)
    
    # Should have registered tools with FastMCP
    # We'll verify this by checking that the tools are properly extracted
    metadata = fastmcp_server._downstream.get_metadata()
    assert len(metadata["tools"]) == 2
    assert metadata["tools"][0]["name"] == "query_db"
    assert metadata["tools"][1]["name"] == "update_record"


def test_fastmcp_server_dynamic_resource_registration():
    """Test that FastMCPServer registers resources from downstream metadata."""
    mock_server = MockDictMCPServer({
        "resources": [
            {"uri": "db://users", "name": "users_table"},
            {"uri": "db://orders", "name": "orders_table"}
        ]
    })
    
    fastmcp_server = FastMCPServer(mock_server)
    
    # Should have registered resources with FastMCP
    metadata = fastmcp_server._downstream.get_metadata()
    assert len(metadata["resources"]) == 2
    assert metadata["resources"][0]["uri"] == "db://users"
    assert metadata["resources"][1]["uri"] == "db://orders"


def test_fastmcp_server_handles_empty_metadata():
    """Test FastMCPServer handles empty metadata gracefully."""
    mock_server = MockDictMCPServer({})
    
    # Should not fail with empty metadata
    fastmcp_server = FastMCPServer(mock_server)
    
    assert fastmcp_server is not None
    metadata = fastmcp_server._downstream.get_metadata()
    assert metadata == {}


def test_fastmcp_server_has_run_method():
    """Test that FastMCPServer has a run method for starting the server."""
    mock_server = MockDictMCPServer()
    fastmcp_server = FastMCPServer(mock_server)
    
    # Should have a run method (we'll mock the actual execution)
    assert hasattr(fastmcp_server, 'run')
    assert callable(fastmcp_server.run)


def test_fastmcp_server_tool_registration_with_missing_description():
    """Test tool registration when description is missing."""
    mock_server = MockDictMCPServer({
        "tools": [
            {"name": "tool_no_desc"}  # Missing description
        ]
    })
    
    # Should handle missing description gracefully
    fastmcp_server = FastMCPServer(mock_server)
    
    assert fastmcp_server is not None
    metadata = fastmcp_server._downstream.get_metadata()
    assert metadata["tools"][0]["name"] == "tool_no_desc"


def test_fastmcp_server_resource_registration_with_missing_name():
    """Test resource registration when name is missing."""
    mock_server = MockDictMCPServer({
        "resources": [
            {"uri": "test://resource"}  # Missing name
        ]
    })
    
    # Should handle missing name gracefully (using URI as name)
    fastmcp_server = FastMCPServer(mock_server)
    
    assert fastmcp_server is not None
    metadata = fastmcp_server._downstream.get_metadata()
    assert metadata["resources"][0]["uri"] == "test://resource"


def test_fastmcp_server_run_filters_name_parameter():
    """Test that run() method filters out 'name' parameter."""
    mock_server = MockDictMCPServer()
    FastMCPServer(mock_server, name="test-server")
    
    # Test the filtering logic directly
    kwargs = {"transport": "stdio", "name": "should-be-filtered", "port": 8080}
    filtered = {k: v for k, v in kwargs.items() if k != 'name'}
    
    assert "name" not in filtered
    assert "transport" in filtered
    assert "port" in filtered
    assert filtered["transport"] == "stdio"
    assert filtered["port"] == 8080


def test_fastmcp_server_tool_handler_execution():
    """Test that tool handlers delegate to downstream server correctly."""
    responses = []
    
    class MockServerWithCapture(MockDictMCPServer):
        def handle_request(self, request: Dict[str, Any]) -> Dict[str, Any]:
            responses.append(request)
            return {"result": "tool_executed", "tool": request["params"]["name"]}
    
    mock_server = MockServerWithCapture({
        "tools": [{"name": "test_tool", "description": "Test tool"}]
    })
    
    FastMCPServer(mock_server)
    
    # Simulate tool execution by accessing the registered tool handler
    # Note: In real usage, FastMCP would handle this, but we test the logic
    test_request = {
        "method": "tools/call",
        "params": {
            "name": "test_tool",
            "arguments": {"param1": "value1"}
        }
    }
    
    response = mock_server.handle_request(test_request)
    
    assert response["result"] == "tool_executed"
    assert response["tool"] == "test_tool"
    assert len(responses) == 1
    assert responses[0]["method"] == "tools/call"


def test_fastmcp_server_resource_handler_execution():
    """Test that resource handlers delegate to downstream server correctly."""
    responses = []
    
    class MockServerWithCapture(MockDictMCPServer):
        def handle_request(self, request: Dict[str, Any]) -> Dict[str, Any]:
            responses.append(request)
            return {"result": "resource_read", "uri": request["params"]["uri"]}
    
    mock_server = MockServerWithCapture({
        "resources": [{"uri": "test://resource", "name": "test_resource"}]
    })
    
    FastMCPServer(mock_server)
    
    # Simulate resource access by accessing the registered resource handler
    test_request = {
        "method": "resources/read",
        "params": {
            "uri": "test://resource"
        }
    }
    
    response = mock_server.handle_request(test_request)
    
    assert response["result"] == "resource_read"
    assert response["uri"] == "test://resource"
    assert len(responses) == 1
    assert responses[0]["method"] == "resources/read"


def test_fastmcp_server_custom_name():
    """Test that FastMCPServer accepts custom name."""
    mock_server = MockDictMCPServer()
    fastmcp_server = FastMCPServer(mock_server, name="custom-server-name")
    
    assert fastmcp_server is not None
    # The name is passed to FastMCP constructor, we can't easily test it
    # but we verify the server initializes correctly